#include "format.h"
#include "palette.h"
#include "png++/png.hpp"
#include "lyra/lyra.hpp"

#include <iterator>

int main(int argc, char **argv) {

	// Parse command arguments

	std::string source, destination;
	bool show_help = false;
	auto cli = lyra::cli_parser();

	cli |= lyra::help(show_help);
	cli |= lyra::arg(source, "source")("Source lev file").required();
	cli |= lyra::arg(destination, "destination")("Destination png file").required();

	auto result = cli.parse({argc, argv});

	if (!result) {
		std::cerr << "Error: " << result.errorMessage() << std::endl;
		std::cerr << cli << std::endl; 
		exit(1);
	}

	if (show_help) {
		std::cout << cli << std::endl;
		exit(0);
	}

	// Verify lev size

	const int width = 504;
	const int height = 350;

    std::ifstream file(source, std::ios::binary);
    file.unsetf(std::ios::skipws);
    std::streampos fileSize;
    file.seekg(0, std::ios::end);
    fileSize = file.tellg();
    file.seekg(0, std::ios::beg);
    
    if(fileSize < width*height) {
        std::cerr << "Error loading level '" << source <<
            "': [Filesize < " << (width*height) << " bytes]" << std::endl;
        return -1;
    }

	// Get lev data

    std::vector<uint8_t> data;
    data.clear();
    data.reserve(fileSize);

    data.insert(data.begin(),
               std::istream_iterator<uint8_t>(file),
               std::istream_iterator<uint8_t>());
    
	// Chop off remainder

    data.resize(width*height);

	// Load palette

	png::palette liero_palette(256);

	for(int i = 0; i<256; i++) {
		liero_palette[i] = png::color(lpl_bin[i*3], lpl_bin[i*3+1], lpl_bin[i*3+2]);
	}

	// Copy and write png

	png::image<png::index_pixel> image(width, height);
	image.set_palette(liero_palette);
	
	for (uint32_t y = 0; y < image.get_height(); y++) {
		for (uint32_t x = 0; x < image.get_width(); x++) {
			image.set_pixel(x,y,data[y*width+x]);
		}
	}
	image.write(destination);
}